/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import java.util.concurrent.ArrayBlockingQueue;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstrakte Versandkomponente. Enth&auml;lt die Hauptschleife, die Logik des
 * Versands und eine abstrakte Versandmethode.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public abstract class AbstraktKexVersandKomponente extends Thread {

	/**
	 * Max. Gr&ouml;sse der Warteschlange.
	 */
	private int maxWarteschlangeGroesse;

	/**
	 * Die Warteschlange.
	 */
	private ArrayBlockingQueue<IVersandAuftrag> warteSchlange = new ArrayBlockingQueue<IVersandAuftrag>(1);

	/**
	 * Anzahl der Versuche bei Misserfolg.
	 */
	private int maxVersuche;

	/**
	 * L&auml;nge der Pause zwischen den Versuchen.
	 */
	private long pause;

	/**
	 * Logger.
	 */
	private Debug logger = Debug.getLogger();

	/**
	 * . 
	 * @return {@link Debug}
	 */
	public Debug getLogger() {
		return logger;
	}

	/**
	 * Beendet die Endlosschleife.
	 */
	private boolean beenden;

	/**
	 * Verbindung zum DAV.
	 */
	private ClientDavInterface dav;

	/**
	 * .
	 * @return {@link ClientDavInterface}
	 */
	public ClientDavInterface getDav() {
		return dav;
	}

	/**
	 * Bearbeitet einen Versandauftrag und stellt ihn in die Warteschlange.
	 *
	 * @param auftrag
	 *            Versandauftrag
	 * @return Anzahl der Auftr&auml;ge in der Wartschlange, wenn gleich
	 *         maxWarteschlangeGroesse+1, dann ist die Warteschlange voll und
	 *         der Auftrag wird nicht bearbeitet
	 */
	public int bearbeite(final IVersandAuftrag auftrag) {
		if (warteSchlange.size() >= maxWarteschlangeGroesse) {
			return maxWarteschlangeGroesse + 1;
		}
		try {
			warteSchlange.put(auftrag);
			LogTools.log(logger, EmailFaxSmsMsg.FineAuftragEingefuegt, auftrag.getAppID(), auftrag.getLaufNr(),
					auftrag.getClass().getName());
		} catch (final InterruptedException e) {
			LogTools.log(logger, EmailFaxSmsMsg.WarningWarteschlangeAbbruch, e.getMessage());
		}

		return warteSchlange.size();
	}

	/**
	 * Gibt die aktuelle Gr&ouml;&szlig;e der Warteschlange zur&uuml;ck.
	 *
	 * @return aktuelle Gr&ouml;&szlig;e der Warteschlange
	 */
	public int getGroesse() {
		return warteSchlange.size();
	}

	/**
	 * Gibt die maximale Gr&ouml;&szlig;e der Warteschlange zur&uuml;ck.
	 *
	 * @return Maximale Gr&ouml;&szlig;e der Warteschlange
	 */
	public int getMaxGroesse() {
		return maxWarteschlangeGroesse;
	}

	/**
	 * Initialisierung des Versandmoduls.
	 *
	 * @param connection
	 *            Verbindung zum DAV
	 * @param modul
	 *            Versandmodul
	 * @param maxWarteschlange
	 *            max. Gr&ouml;sse der Warteschlange
	 */
	public void init(final ClientDavInterface connection, final SystemObject modul, final int maxWarteschlange) {
		dav = connection;
		warteSchlange = new ArrayBlockingQueue<IVersandAuftrag>(maxWarteschlange);
		this.maxWarteschlangeGroesse = maxWarteschlange;
	}

	/**
	 * Paramtriert das Versandmodul.
	 *
	 * @param versuche
	 *            Anzahl der Versandversuche
	 * @param pausenzeit
	 *            Pause zwischen den Versuchen in Sekunden
	 */
	public void parametriereGenerell(final int versuche, final long pausenzeit) {
		this.maxVersuche = versuche;
		this.pause = pausenzeit * 1000L;
	}

	/**
	 * Endloseschleife zur Abarbeitung der Auftr&auml;ge.
	 */
	@Override
	public void run() {
		while (!beenden) {
			try {
				IVersandAuftrag auftrag;
				auftrag = warteSchlange.take();
				// Test, ob Versuche parametriert sind
				if (maxVersuche == 0) {
					logger.warning("Keine Anzahl Wiederholungen parametriert");
					continue;
				}

				final VersandStatus status = sendeAuftrag(auftrag, maxVersuche, pause);
				VersandStatusSender.getInstanz().send(auftrag, status);
			} catch (final InterruptedException e) {
				LogTools.log(logger, EmailFaxSmsMsg.InfoVersandUnterbrochen, e.getMessage());
				if (beenden) {
					return;
				}
			}
		}
	}

	/**
	 * Abstrakte Versandmethode.
	 *
	 * @param auftrag
	 *            Der Auftrag
	 * @param versuche
	 *            Anzahl der Versuche
	 * @param pausezeit
	 *            Pause zwischen den Versandversuchen in Milisekunden
	 * @return Status des Versandes
	 * @throws InterruptedException
	 *             Wenn der Versand unterbrochen wurde
	 */
	protected abstract VersandStatus sendeAuftrag(IVersandAuftrag auftrag, int versuche, long pausezeit)
			throws InterruptedException;

	/**
	 * Testet, ob die Komponente beendet werden soll.
	 *
	 * @return true, wenn die terminate Methode gerufen wurde
	 */
	protected boolean sollBeenden() {
		return beenden;
	}

	/**
	 * Beendet den Versandmodul.
	 */
	public void terminate() {
		beenden = true;
		this.interrupt();
	}

}
