/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms.email.smtp;

import java.util.Collection;
import java.util.Date;
import java.util.Properties;

import javax.mail.AuthenticationFailedException;
import javax.mail.Message;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import com.sun.mail.smtp.SMTPTransport;

import de.bsvrz.kex.emailfaxsms.VersandStatus;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Versendet Emails, ein Wrapper f&uuml;r die JavaMailAPI.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class KexEmailSender {

	/**
	 * Debug Status.
	 */
	protected static boolean dEBUG = false;
	/**
	 * LOGGER.
	 */
	private static final Debug LOGGER = Debug.getLogger();
	/**
	 * Session mit dem SMTP Server.
	 */
	protected Session session;
	/**
	 * Ob man sich Authentifizieren soll.
	 */
	private boolean authReq = false;
	/**
	 * Authentifizierung.
	 */
	private PasswordAuthentication authentifizierung;
	/**
	 * Adresse des MailServers.
	 */
	private final String mailServer;

	/**
	 * Standardkonstruktor.
	 *
	 * @param mailServer
	 *            Addresse des EmailServers
	 * @param user
	 *            Username
	 * @param passwort
	 *            Passwort
	 * @param authReq
	 *            true, wenn Authentifizierung benoetigt ist
	 * @param starttls
	 *            unklar
	 */
	public KexEmailSender(final String mailServer, final String user, final String passwort, final boolean authReq,
			final boolean starttls) {

		final Properties props = new Properties();
		if (authReq) {
			this.authReq = true;
			if (KexEmailSender.dEBUG) {
				props.put("mail.smtp.debug", "true");
			}
			if (authReq) {
				props.put("mail.smtp.auth", "true");
				props.put("mail.smtp.starttls.enable", "true");
			}
			authentifizierung = new PasswordAuthentication(user, passwort);
		}
		this.mailServer = mailServer;
		props.put("mail.smtp.host", mailServer);
		session = Session.getInstance(props, null);
		if (KexEmailSender.dEBUG) {
			session.setDebug(true);
		}
	}

	/**
	 * Versendet eine Email.
	 *
	 * @param absender
	 *            Absender
	 * @param betreff
	 *            Betreff
	 * @param text
	 *            Text
	 * @param empfaenger
	 *            Hauptempfaenger
	 * @param ccList
	 *            Empfengerliste fuer Kopie
	 * @param bccList
	 *            Empfengerliste fuer Kopie, ohne Benachrichtigung ueber anderen
	 *            Empfaenger
	 * @return VersandStatus
	 */
	public VersandStatus send(final String absender, final String betreff, final String text, final String empfaenger,
			final Collection<String> ccList, final Collection<String> bccList) {

		VersandStatus status = VersandStatus.NICHT_VERSENDET;

		try {
			final Message msg = new MimeMessage(session);

			msg.setFrom(new InternetAddress(absender));
			msg.setRecipient(Message.RecipientType.TO, new InternetAddress(empfaenger, true));

			final InternetAddress[] ccAdress = new InternetAddress[ccList.size()];
			final InternetAddress[] bccAdress = new InternetAddress[bccList.size()];

			int i = 0;
			for (final String adr : ccList) {
				ccAdress[i++] = new InternetAddress(adr, true);
			}

			i = 0;
			for (final String adr : bccList) {
				bccAdress[i++] = new InternetAddress(adr, true);
			}

			msg.setRecipients(Message.RecipientType.CC, ccAdress);
			msg.setRecipients(Message.RecipientType.BCC, bccAdress);
			msg.setSubject(betreff);
			msg.setText(text);

			msg.setHeader("X-Mailer", "KexEmailSender");
			msg.setSentDate(new Date());
			msg.saveChanges();

			final SMTPTransport t = (SMTPTransport) session.getTransport("smtp");
			t.setReportSuccess(false);

			try {
				if (authReq) {
					t.connect(mailServer, authentifizierung.getUserName(), authentifizierung.getPassword());
				} else {
					t.connect();
				}
				t.sendMessage(msg, msg.getAllRecipients());
			} catch (final AuthenticationFailedException se) {
				final String s = "Email nicht versendet: Authentifizierungsfehler am Mailserver";
				KexEmailSender.LOGGER.warning(s);
				MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.WARNING, s);
				return new VersandStatus("Fehler beim Versand: " + s, false);
			} catch (final Exception se) {
				final String s = "Email nicht versendet: " + se.getMessage();
				KexEmailSender.LOGGER.warning(s, se);
				MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.WARNING, s);
				return new VersandStatus("Fehler beim Versand: " + s, false);
			} finally {
				t.close();
				status = VersandStatus.VERSAND_OK;
			}
		} catch (final Exception e) {
			final String s = "Email nicht versendet: " + e.getMessage();
			KexEmailSender.LOGGER.warning(s, e);
			MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.WARNING, s);
			return new VersandStatus("Fehler beim Versand: " + e.getMessage(), false);
		}

		return status;
	}
}
