/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007, 2015 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.emailfaxsms.voip;

import java.io.IOException;
import java.net.HttpURLConnection;

import de.bitctrl.sibacop.client.AbstractRequest;
import de.bitctrl.sibacop.client.Connection;
import de.bitctrl.sibacop.client.ExitRequest;
import de.bitctrl.sibacop.client.SocketConnection;
import de.bsvrz.kex.emailfaxsms.VersandStatus;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Versendet Nachrichten &uuml;ber VoIP.
 *
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: KexVoIPSender.java 63847 2016-10-17 11:36:14Z uhlmann $
 */
public abstract class KexVoIPSender {

	/**
	 * LOGGER.
	 */
	protected static final Debug LOGGER = Debug.getLogger();

	/** Die Response-Code sind HTTP-Codes. */
	public static final int VOIP_RESPONSE_CODE_NO_ERROR = HttpURLConnection.HTTP_OK;
	
	/** Answered.	*/
	public static final int VOIP_RESPONSE_CODE_ANSWERED = 211;
	
	/** Completed.	*/
	public static final int VOIP_RESPONSE_CODE_COMPLETED = 212;
	

	/**
	 * Host.
	 */
	private final String host;
	/**
	 * Port.
	 */
	private final int port;

	/**
	 * Standardkonstruktor.
	 *
	 * @param host
	 *            VoIP-Sender Host
	 * @param port
	 *            VoIP-Sender Port
	 * 
	 * @throws Exception
	 *             bei Fehlern
	 */
	public KexVoIPSender(final String host, final int port) throws Exception {
		this.host = host;
		this.port = port;

		KexVoIPSender.LOGGER.fine("VoIP-Sender Initialisierung komplett");
	}

	/**
	 * Sendet eine Nachricht (Voicemail oder FAX) über VoiP.
	 *
	 * @param empfaenger
	 *            Empf&auml;nger
	 * @param text
	 *            Text in Latin
	 * @param debug
	 *            Debug-Flag
	 * @return VersandStatus
	 */
	public VersandStatus send(final String empfaenger, final String text, final boolean debug) {
		Connection connection = null;
		try {
			LOGGER.fine("Verbindung wird aufgebaut zu Host: " + host + ", Port: " + port);
			connection = SocketConnection.create(host, port);
			connection.setVerbose(debug);
			LOGGER.fine("Request wird erzeugt");
			final AbstractRequest<?>  req = erzeugeRequest(connection, empfaenger, text);
			LOGGER.fine("Request wird gesendet");
			req.send();
			final int responseCode = req.getResponseCode();
			LOGGER.fine("Request-Response-Code: " + responseCode);
			
			if (isOkResponse(responseCode)) {
				return VersandStatus.VERSAND_OK;
			}

			final String s = "Fehler beim Versand einer VoiP-Nachricht: Response code " + responseCode + ": " 
					+ req.getResponseMessage() + " " + req.getResponseData();
			MessageSender.getInstance().sendMessage(MessageType.SYSTEM_DOMAIN, MessageGrade.WARNING, s);
			return new VersandStatus(s, false);
		} catch (final IOException e) {
			final String s = "VoIP-Nachricht nicht versendet: " + e.getLocalizedMessage();
			LOGGER.warning(s, e);
			MessageSender.getInstance().sendMessage(MessageType.SYSTEM_DOMAIN, MessageGrade.WARNING, s);
			return new VersandStatus(s, false);
		} finally {
			if (connection != null) {			
				LOGGER.fine("ExitRequest wird erzeugt");
				final ExitRequest req = connection.createRequest(ExitRequest.class);
				try {
					LOGGER.fine("ExitRequest wird gesendet");
					req.send();
					LOGGER.fine("Verbindung wird geschlossen");
					connection.close();
				} catch (final IOException e) {
					LOGGER.warning("Versand VoIP-Nachricht Verbindungsabbau: " + e.getMessage());
				}
			}
		}
	}

	protected abstract boolean isOkResponse(int responseCode);

	protected abstract AbstractRequest<?> erzeugeRequest(final Connection connection, final String empfaenger, final String text);
}
